/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System
**   07 March 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

/** \file
 **   
 ** This file implements the adc driver for the mega128/STK500
 ** environment.
 **
 ** It can be used as a template for other environments.
 ** 
 ** The implementation relies heavily on a project specific config.h
 ** file that must be included at the same time, to be useful.
 **
 ********************************************************************/

#ifndef __DRV_ADC_H__
#define __DRV_ADC_H__

#include <avr/io.h>

#include "config.h"


/**
 ** \brief The value type delivered by the ADC.
 */
typedef union {
  uint16_t uvalue16b;
  int16_t  value16b;
  struct {
    unsigned int uvalue10b:10;
    unsigned int foo_:6;
  };
  struct {
    signed int   value10b:10;
    unsigned int foo_:6;
  };
} HUS_adc_value_t;

/**
 ** \brief The value type of the status maintained by the HUS_adc_*
 ** funcs.
 */
typedef struct {
  unsigned ready:1;
  HUS_adc_value_t value;
} HUS_adc_status_t;

/**
 ** \brief Turns the Adc on.
 */
static inline void HUS_adc_on() { ADCSRA |= _BV(ADEN); }

/**
 ** \brief Turns the Adc off.
 */
static inline void HUS_adc_off() { ADCSRA &= ~_BV(ADEN); }

/**
 ** \brief Starts converting.
 */
static inline void HUS_adc_start() { ADCSRA |= _BV(ADSC); }

/**
 ** \brief Enables the free run mode.
 **
 ** Use HUS_adc_start() to start converting.
 */
static inline void HUS_adc_enable_free_run() { ADCSRA |= _BV(ADFR); }

/**
 ** \brief Stops the free run mode.
 */
static inline void HUS_adc_stop_free_run() { ADCSRA &= ~_BV(ADFR); }

/**
 ** \brief Enables the ADC Conversion Complete interrupt.
 */
static inline void HUS_adc_enable_intr() { ADCSRA |= _BV(ADIE); }

/**
 ** \brief Disables the ADC Conversion Complete interrupt.
 */
static inline void HUS_adc_disable_intr() { ADCSRA &= ~_BV(ADIE); }

/**
 ** \brief Reads the ADC result.
 */
static inline HUS_adc_value_t HUS_adc_read() { return (HUS_adc_value_t)ADC;}

/**
 ** \brief Align result left.
 */
static inline void HUS_adc_align_left() { ADMUX |= _BV(ADLAR); }

/**
 ** \brief Align result right.
 */
static inline void HUS_adc_align_right() { ADMUX &= ~_BV(ADLAR); }




/**
 ** \brief Adc Vref initialisation.
 **
 */
#define HUS_ADC_REF_CLEARMASK	(~(_BV(REFS1)|_BV(REFS0)))
#define HUS_ADC_REF_EXTERN	(0)
#define HUS_ADC_REF_AVCC	(_BV(REFS0))
#define HUS_ADC_REF_IVREF	(_BV(REFS1)|_BV(REFS0))
static inline void HUS_adc_vref(unsigned int vref_mask) { 
  ADMUX &= HUS_ADC_REF_CLEARMASK;
  ADMUX |= vref_mask;
}

/**
 ** \brief Adc prescaler initialisation.
 **
 */
#define HUS_ADC_PRESC_CLEARMASK	(~(_BV(ADPS2)|_BV(ADPS1)|_BV(ADPS0)))
#define HUS_ADC_PRESC_0		0
#define HUS_ADC_PRESC_2		(_BV(ADPS0))
#define HUS_ADC_PRESC_4		(_BV(ADPS1))
#define HUS_ADC_PRESC_8		(_BV(ADPS1)|_BV(ADPS0))
#define HUS_ADC_PRESC_16	(_BV(ADPS2))
#define HUS_ADC_PRESC_32	(_BV(ADPS2)|_BV(ADPS0))
#define HUS_ADC_PRESC_64	(_BV(ADPS2)|_BV(ADPS1))
#define HUS_ADC_PRESC_128	(_BV(ADPS2)|_BV(ADPS1)|_BV(ADPS0))

static inline void HUS_adc_prescaler(const unsigned int prescaler_mask) {
  ADCSRA &= HUS_ADC_PRESC_CLEARMASK;
  ADCSRA |= prescaler_mask;
}

/**
 ** \brief Adc hardware initialisation.
 **
 ** Selects the single ended ADC0 pin, with 1x gain, enables the
 ** Conversion Complete interrupt, and sets the callback to be called
 ** by the ISR.
 **
 */
void HUS_adc_init_ADC0();//HUS_adc_callback_t callback);

/**
 ** \brief Adc hardware initialisation.
 **
 ** Selects the differential ADC0/ADC0 pin, with 10x gain, enables the
 ** Conversion Complete interrupt, and sets the callback to be called
 ** by the ISR.
 **
 */
void HUS_adc_init_diff_ADC0_ADC0_x10();//HUS_adc_callback_t callback);

/**
 ** \brief Adc hardware initialisation.
 **
 ** Selects the differential ADC0/ADC1 pin, with 10x gain, enables the
 ** Conversion Complete interrupt, and sets the callback to be called
 ** by the ISR.
 **
 */
void HUS_adc_init_diff_ADC0_ADC1_x10();//HUS_adc_callback_t callback);


/**
 ** \brief Adc hardware initialisation.
 **
 ** Selects the differential ADC0/ADC0 pin, with 200x gain, enables
 ** the Conversion Complete interrupt, and sets the callback to be
 ** called by the ISR.
 **
 */
void HUS_adc_init_diff_ADC0_ADC0_x200();//HUS_adc_callback_t callback);

/**
 ** \brief Adc hardware initialisation.
 **
 ** Selects the differential ADC0/ADC1 pin, with 200x gain, enables
 ** the Conversion Complete interrupt, and sets the callback to be
 ** called by the ISR.
 **
 */
void HUS_adc_init_diff_ADC0_ADC1_x200();//HUS_adc_callback_t callback);


/**
 ** \brief Convert adc value to volts x 10e6.
 **
 ** U = (ADCValue * VRef * 10e6) / 1024
 **
 */
int32_t HUS_adc_se_to_voltx10e6(int32_t adc_value);

/**
 ** \brief Convert adc value to volts x 10e6.
 **
 ** U = (ADCValue * VRef * 10e6) / (512 * ADC_GAIN)
 **
 */
int32_t HUS_adc_dif_to_voltx10e6(int32_t adc_value, uint8_t adc_gain);


#endif /*  __DRV_ADC_H__ */
