/*********************************************************************
**   Martial Chateauvieux
**   HORUS UAV System
**   23 Mai 2007
**********************************************************************
**
**    HORUS UAV System V1.1.0 - Copyright (C) 2003-2009 Martial
**    Chateauvieux
**
**    This file is part of the HORUS UAV System distribution.
**
**    HORUS UAV System is free software; you can redistribute it
**    and/or modify it under the terms of the GNU General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    HORUS UAV System is distributed in the hope that it will be
**    useful, but WITHOUT ANY WARRANTY; without even the implied
**    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**    See the GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public
**    License along with the HORUS UAV System; if not, write to the
**    Free Software Foundation, Inc., 59 Temple Place, Suite 330,
**    Boston, MA 02111-1307 USA
**
*********************************************************************/

#include <inttypes.h>
#include <avr/io.h>

#include "config.h"

#include "macros_io.h"

#include "mydelay.h"
#include "drv_lcd.h"

#define READ_BUSY_FLAG 1

static HUS_lcd_status_t lcd_status;

inline static void _set_RS_DDR() {HUS_IO_SET_OUTP(HUS_LCD_RS);}
inline static void _set_RS_bit() {HUS_IO_SET_HIGH(HUS_LCD_RS);}
inline static void _clr_RS_bit() {HUS_IO_SET_LOW(HUS_LCD_RS);}

inline static void _set_E_DDR()  {HUS_IO_SET_OUTP(HUS_LCD_E);}
inline static void _set_E_bit()  {HUS_IO_SET_HIGH(HUS_LCD_E);}
inline static void _clr_E_bit()  {HUS_IO_SET_LOW(HUS_LCD_E);}

inline static void _set_RW_DDR() {HUS_IO_SET_OUTP(HUS_LCD_RW);}
inline static void _set_RW_bit() {HUS_IO_SET_HIGH(HUS_LCD_RW);}
inline static void _clr_RW_bit() {HUS_IO_SET_LOW(HUS_LCD_RW);}

inline static void _set_data_DDR() {
    HUS_IO_SET_OUTP(HUS_LCD_DB4);
    HUS_IO_SET_OUTP(HUS_LCD_DB5);
    HUS_IO_SET_OUTP(HUS_LCD_DB6);
    HUS_IO_SET_OUTP(HUS_LCD_DB7);
}
inline static void _clr_data_DDR() {
    HUS_IO_SET_INP(HUS_LCD_DB4);
    HUS_IO_SET_INP(HUS_LCD_DB5);
    HUS_IO_SET_INP(HUS_LCD_DB6);
    HUS_IO_SET_INP(HUS_LCD_DB7);
}
inline static void _clr_data() {
    HUS_IO_SET_HIGH(HUS_LCD_DB4);
    HUS_IO_SET_HIGH(HUS_LCD_DB5);
    HUS_IO_SET_HIGH(HUS_LCD_DB6);
    HUS_IO_SET_HIGH(HUS_LCD_DB7);
}

inline static void _write_data(const char c) {
    if (c & 0x01) HUS_IO_SET_HIGH(HUS_LCD_DB4);
    else 	  HUS_IO_SET_LOW(HUS_LCD_DB4);

    if (c & 0x02) HUS_IO_SET_HIGH(HUS_LCD_DB5);
    else          HUS_IO_SET_LOW(HUS_LCD_DB5);

    if (c & 0x04) HUS_IO_SET_HIGH(HUS_LCD_DB6);
    else          HUS_IO_SET_LOW(HUS_LCD_DB6);

    if (c & 0x08) HUS_IO_SET_HIGH(HUS_LCD_DB7);
    else          HUS_IO_SET_LOW(HUS_LCD_DB7);
}
static inline void _lcd_e_toggle( void ) {
    _set_E_bit();
    _clr_E_bit();
}

char HUS_lcd_read(uint8_t read_busy_flag) {
    char tmp = 0;

    _clr_data();
    _clr_data_DDR();

    if (read_busy_flag) _clr_RS_bit();	// Busy flag & Address
    else _set_RS_bit();			// Address
    _set_RW_bit();			// Read

    _set_E_bit();
    if (HUS_LCD_DB4_PIN & _BV(HUS_LCD_DB4_BIT)) tmp |= 0x10;
    if (HUS_LCD_DB5_PIN & _BV(HUS_LCD_DB5_BIT)) tmp |= 0x20;
    if (HUS_LCD_DB6_PIN & _BV(HUS_LCD_DB6_BIT)) tmp |= 0x40;
    if (HUS_LCD_DB7_PIN & _BV(HUS_LCD_DB7_BIT)) tmp |= 0x80;
    _clr_E_bit();

    _set_E_bit();
    if (HUS_LCD_DB4_PIN & _BV(HUS_LCD_DB4_BIT)) tmp |= 0x01;
    if (HUS_LCD_DB5_PIN & _BV(HUS_LCD_DB5_BIT)) tmp |= 0x02;
    if (HUS_LCD_DB6_PIN & _BV(HUS_LCD_DB6_BIT)) tmp |= 0x04;
    if (HUS_LCD_DB7_PIN & _BV(HUS_LCD_DB7_BIT)) tmp |= 0x08;
    _clr_E_bit();

    _set_data_DDR();

    return tmp;
}

static inline void HUS_lcd_waitbusy( void ) {
#if defined(HUS_LCD_USEBUSYPIN)            
    while(HUS_lcd_read(READ_BUSY_FLAG) & _BV(HUS_LCD_BUSY_BIT)); 
#else
    delay_ms(10);
#endif
}

void HUS_lcd_write(const char data, uint8_t cmdflg) {
    HUS_lcd_waitbusy();

    if (cmdflg) _clr_RS_bit();	// CMD
    else _set_RS_bit();		// DATA

    _clr_RW_bit();

    _write_data(data >> 4);
    _lcd_e_toggle();

    _write_data(data & 0x0F);
    _lcd_e_toggle();
}


/* Function set bits */
#define __HUS_LCD_FUNC_SET_CMD_BIT _BV(5)
#define __HUS_LCD_DH_BIT		 _BV(0)	// 0=scroll, 1=shift
#define __HUS_LCD_REV_BIT		 _BV(1)	// 1=Video reversed
#define __HUS_LCD_RE_BIT		 _BV(2)	// Extension register
#define __HUS_LCD_N_BIT		 _BV(3)	// 0= 1 line, 1=2 lines
#define __HUS_LCD_DL_BIT		 _BV(4)	// 0=4bits intf., 1=8bits intf.


/* Extended Function set bits */
#define __HUS_LCD_EXT_FUNC_SET_CMD_BIT _BV(3)
#define __HUS_LCD_NW_BIT		     _BV(0)	// 0=1- or 2- line, 1=4-line display
#define __HUS_LCD_BW_BIT		     _BV(1)	// 0=normal cursor, 1=inverting cursor
#define __HUS_LCD_FW_BIT		     _BV(2)	// 0=5dot font, 1=6dot font

#if defined HUS_LCD_CONTROLLER_KS0073
void HUS_lcd_init(const HUS_lcd_type_t *lcd_type) {

  /* Configure the port as an output */
    _set_data_DDR();
    _set_RS_DDR();
    _set_E_DDR();
    _set_RW_DDR();

    _clr_RS_bit();
    _clr_E_bit();
    _clr_RW_bit();
    _clr_data();
    
    /* Wait for power up */
    delay_ms(16);

    /* Initial write */
    _write_data((char)0x03);
    _lcd_e_toggle();
    delay_ms(5);
    _lcd_e_toggle();
    delay_us(120);
    _lcd_e_toggle();
    delay_us(40);

    /* Select 4 bit mode */
    _write_data((char)0x02);
    _lcd_e_toggle();

    /* from now the LCD only accepts 4 bit I/O, we can use HUS_lcd_cmd() */    

    /* Turn extended function on: set the RE Bit */
    HUS_lcd_cmd(__HUS_LCD_FUNC_SET_CMD_BIT | __HUS_LCD_RE_BIT);

    /* Turn 4 line mode on */
    HUS_lcd_cmd(__HUS_LCD_EXT_FUNC_SET_CMD_BIT | __HUS_LCD_NW_BIT);

    /* Turn extended function off: clear the RE Bit */
    HUS_lcd_cmd(__HUS_LCD_FUNC_SET_CMD_BIT);
    
    HUS_lcd_off();			// Nothing on -> off
    HUS_lcd_cls();			// Clear
    HUS_lcd_cmd(HUS_LCD_MODE_INC);	// Increment and do not shift

    lcd_status.pos = 0;
    lcd_status.type.rows = 4;
    lcd_status.type.cols = 20;
  
}
#elif defined HUS_LCD_CONTROLLER_HD44780
void HUS_lcd_init(const HUS_lcd_type_t *lcd_type) {
	/* Configure the port as an output */
    _set_data_DDR();
    _set_RS_DDR();
    _set_E_DDR();
    _set_RW_DDR();

    _clr_RS_bit();
    _clr_E_bit();
    _clr_RW_bit();
    _clr_data();
    
    delay_ms(16);
    
    _write_data((char)0x02); // Set 4 bit interface
    _lcd_e_toggle();
    delay_us(50);
    _lcd_e_toggle();

    delay_us(50);
    
    /* Set the RE Bit */
    HUS_lcd_cmd(__HUS_LCD_FUNC_SET_CMD_BIT | __HUS_LCD_RE_BIT);
    
    /* Call the extended function set */
    HUS_lcd_cmd(__HUS_LCD_EXT_FUNC_SET_CMD_BIT | __HUS_LCD_NW_BIT);
    
    /* Function Set */
    HUS_lcd_cmd(__HUS_LCD_FUNC_SET_CMD_BIT);	// 4 bits, *-line, RE=0, scroll

    HUS_lcd_off();			// Nothing on -> off
    HUS_lcd_cls();			// Clear
    HUS_lcd_cmd(HUS_LCD_MODE_INC);	// Increment and do not shift

    lcd_status.pos = 0;
    lcd_status.type.rows = lcd_type->rows;
    lcd_status.type.cols = lcd_type->cols;
}
#endif

extern void HUS_lcd_goto(const uint8_t row, const uint8_t col) {
    lcd_status.pos = (row * lcd_status.type.cols) + col;
    switch(lcd_status.type.rows) {
        case 2:
            HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(row * HUS_LCD_LINE_RAMLEN_2ROWS + col));
            break;
        case 4:
        default:
	  //HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(row * HUS_LCD_LINE_RAMLEN_4ROWS + col));
            switch(row) {	    
	    case 0:
	      HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(HUS_LCD_ROW_0_DDRAMADR + col));
	      break;
	    case 1:
	      HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(HUS_LCD_ROW_1_DDRAMADR + col));
	      break;
	    case 2:
	      HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(HUS_LCD_ROW_2_DDRAMADR + col));
	      break;
	    case 3:
	      HUS_lcd_cmd(HUS_LCD_DDRAM_CMD|(HUS_LCD_ROW_3_DDRAMADR + col));
	      break;
	    }
            break;
    }
}

extern int HUS_lcd_putc(const char c, FILE *stream) {

    if ((lcd_status.pos % lcd_status.type.cols) == 0) {
    	HUS_lcd_goto(lcd_status.pos / lcd_status.type.cols, 0);
    }

    if (c == '\n') lcd_status.pos = 0;
    else {
	   HUS_lcd_data(c);
	   lcd_status.pos++;
    }

    return 0;
}

void HUS_lcd_puts(const char *s, FILE *stream) {
    uint16_t i = 0;
    while(s[i]) { HUS_lcd_putc(s[i++], stream); }
}

void HUS_lcd_puts2(const char *s, FILE *stream) {
    uint16_t i = 0;
    while(s[i]) { HUS_lcd_putc(s[i++], stream); }
}

void HUS_lcd_putsn(const char *s, int len, FILE *stream) {
    uint16_t i = 0;
    while(s[i] && (i < len)) { HUS_lcd_putc(s[i++], stream); }
}

void HUS_lcd_puts_P(PGM_P s, FILE *stream) {
    char c;
    while((c = pgm_read_byte(s++))) { HUS_lcd_putc(c, stream); }
}


void HUS_lcd_on() {
    HUS_lcd_cmd(HUS_LCD_ON_DSPL);
}
void HUS_lcd_off() {
    HUS_lcd_cmd(HUS_LCD_ON_OFF);
}
void HUS_lcd_cls() {
    HUS_lcd_cmd(HUS_LCD_CLR);
    lcd_status.pos = 0;
}

